//===================================================================================================
// Uniform Variables
//===================================================================================================
float4x4 WorldViewProj;
float4x4 World;
float4x4 WorldInverseTranspose;

float4x4 EyePostion;
float4x4 EyePostion1;
float4x4 EyePostion2;
float4x4 EyePostion3;
int ScreenWidth;
int ScreenHeight;

float4x4 light0;
float4x4 light1;
float coreShininess;

float WaveRadius;
float4 boulderLookVector;
float3 boulderEnergyValue;

float4 coreMaterialDiffuse;
float4 coreMaterialSpecular;

float3 halfwayVector1;
float3 halfwayVector2;
float diffuse0;
float diffuse1;
float specular0;
float specular1;
float4 ambientColor;
float4 diffuseColor;
float4 specularColor;
float4 textureValue;

//Blending Variables
float worldMinY;
float worldMaxY;

//===================================================================================================
// Textures and Samplers
//===================================================================================================

texture modelTexture1;
sampler modelTextureSampler1 = sampler_state 
{ 
	texture = <modelTexture1> ; 
    AddressU  = MIRROR;	AddressV  = MIRROR;	AddressW  = MIRROR;
	magfilter = LINEAR; minfilter = LINEAR;	mipfilter=LINEAR; 
};

texture modelTexture2;
sampler modelTextureSampler2 = sampler_state 
{
    texture = <modelTexture2>;
    AddressU  = MIRROR;	AddressV  = MIRROR;	AddressW  = MIRROR;
    MIPFILTER = LINEAR;	MINFILTER = LINEAR;	MAGFILTER = LINEAR;
};

texture modelTexture3;
sampler modelTextureSampler3 = sampler_state 
{
    texture = <modelTexture3>;
    AddressU  = MIRROR;	AddressV  = MIRROR;	AddressW  = MIRROR;
    MIPFILTER = LINEAR;	MINFILTER = LINEAR;	MAGFILTER = LINEAR;
};

texture modelTexture4;
sampler modelTextureSampler4 = sampler_state 
{
    texture = <modelTexture4>;
    AddressU  = MIRROR;	AddressV  = MIRROR;	AddressW  = MIRROR;
    MIPFILTER = LINEAR;	MINFILTER = LINEAR;	MAGFILTER = LINEAR;
};
texture bumpTexture1;
sampler bumpTextureSampler1 = sampler_state 
{
    texture = <bumpTexture1>;
    AddressU  = MIRROR;	AddressV  = MIRROR;	AddressW  = MIRROR;
    MIPFILTER = LINEAR;	MINFILTER = LINEAR;	MAGFILTER = LINEAR;
};
//===================================================================================================
// Uniform, Varying, and FrameBuffer "Stream" Structures - 
// Used to Communicate between CPU, VertexProcessor, FragmentProcessor, and FrameBuffer
//===================================================================================================
struct VertexInput 
{
    float4	position			: POSITION0;		// Position
	float	blendWeight			: BLENDWEIGHT0; 	// Blend weights
	float4	blendIndices		: BLENDINDICES0;	// Blend indices
    float3	normal				: NORMAL;			// Normal vector
	float	pointSize			: PSIZE0;			// Point size
    float4	color				: COLOR0;			// Color
    float4	texCoord			: TEXCOORD0;		// Texture coordinates
    float3	tangent				: TANGENT0;			// Tangent
    float3	binormal			: BINORMAL0;		// Binormal
	float	tessFactor			: TESSFACTOR0; 		// Tessellation factor	
};

struct VertexInternal
{
    // Location
    float3 lastPosition;
    float3 currentPosition;
    float3 referencePosition;

    // Orientation
    half normalVectorZenith;		    // Half types are 16-bit floating point
    half normalVectorAzimuth;
    half upVectorZenith;
    half upVectorAzimuth;
    half perpVectorZenith;
    half perpVectorAzimuth;

    // Physical motion 
    float3 velocity;
    float3 totalForce;
    half scale;
    half mass;
    half adhesion;
    half cohesion;
    half energy;
    half stateChangeThreshold;
};
struct VertexOutput
{
	// These values are used directly by the rasterizer
	float4	position			: POSITION; 		// Position
	float	pointSize			: PSIZE; 			// Point size
	float	fog					: FOG; 				// Vertex fog

	// These values are interpolated and sent to FragIn
	float4	color				: COLOR0; 			// Color (example: COLOR0)
    float3	vertexToEyeVector	: COLOR1;

	float4	texCoord			: TEXCOORD0; 		// Texture coordinates (example: TEXCOORD0)
    float3	normal 				: TEXCOORD1;		// Texture coordinates 
    float3	worldPos  			: TEXCOORD2;		// Texture coordinates 
    float3	lightVector1 		: TEXCOORD3;		// Texture coordinates 	
	float3	lightVector2		: TEXCOORD4;

	// Free Variables (limit)	
	float3	lightVector3		: TEXCOORD5;
	float3	lightVector4		: TEXCOORD6;
	float3	lightVector5		: TEXCOORD7;

};

struct FragmentInput
{
	float4	color				: COLOR0; 			// Color (example: COLOR0)
    float3	vertexToEyeVector	: COLOR1;

    float4	texCoord			: TEXCOORD0;		// Texture coordinates 
    float3	normal				: TEXCOORD1;		// Texture coordinates 
    float3	worldPos			: TEXCOORD2;		// Texture coordinates 
    float3	lightVector1		: TEXCOORD3;		// Texture coordinates 
	float3	lightVector2		: TEXCOORD4;
	
	// Free Variables (limit)
	float3	lightVector3		: TEXCOORD5;
	float3	lightVector4		: TEXCOORD6;
	float3	lightVector5		: TEXCOORD7;

};
struct FragmentInternal{

};
struct FragmentOutput
{
	float4	color				: COLOR0; 			// Color for render target n
	//float	depth				: DEPTH; 			// Depth value
};


//===================================================================================================
// Vertex and Fragment Shader
//===================================================================================================

// Core Light Shader
//--------------------------------------------------------------------------------------------------
FragmentOutput FS_BasicLighting(FragmentInput IN, FragmentOutput OUT)
{
	// Calculate the pixel to light source vectors
	IN.lightVector1				-= IN.worldPos;																// Vertex to Light source 1 vector 
	IN.lightVector2				-= IN.worldPos;																// Vertex to Light source 2 vector 

	// Calculate each light source's halfway vector
	float3 halfwayVector1		= normalize(IN.vertexToEyeVector + IN.lightVector1);						// Halfway vector
  	float3 halfwayVector2		= normalize(IN.vertexToEyeVector + IN.lightVector2);						// Halfway vector

	// Calculate the diffuse and specular contributions
    float diffuse0				= max(0, dot(normalize(IN.normal), normalize(IN.lightVector1)));			// Diffuse component
    float diffuse1				= max(0, dot(normalize(IN.normal), normalize(IN.lightVector2)));			// Diffuse component
    float specular0				= diffuse0 * pow(dot(normalize(IN.normal), halfwayVector1), coreShininess);	// Specular component
    float specular1				= diffuse1 * pow(dot(normalize(IN.normal), halfwayVector2), coreShininess);	// Specular component

	// Calculate Ambient, Diffuse, and Specular color components
    float4 ambientColor = coreMaterialDiffuse	* (				float4(light0[1][0], light0[1][1], light0[1][2], light0[1][3]) +				float4(light1[1][0], light1[1][1], light1[1][2], light1[1][3]));
    float4 diffuseColor = coreMaterialDiffuse	* (diffuse0 *	float4(light0[2][0], light0[2][1], light0[2][2], light0[2][3]) + diffuse1 *		float4(light1[2][0], light1[2][1], light1[2][2], light1[2][3]) );
    float4 specularColor = coreMaterialSpecular * (specular0 *	float4(light0[3][0], light0[3][1], light0[3][2], light0[3][3]) + specular1 *	float4(light1[3][0], light1[3][1], light1[3][2], light1[3][3]) );

	// Get texture value
 	float4 textureValue =	tex2D(modelTextureSampler1, IN.texCoord); 	

	// Apply lighting to the color
 	OUT.color += (diffuseColor * textureValue) + (ambientColor * textureValue) + specularColor;
 	
 	// To frambuffer
 	return OUT;
}

// Particle Shader
//----------------------------------------------------------------------------------------------------------------
FragmentOutput FS_Particle(FragmentInput IN, FragmentOutput OUT)
{
	OUT.color.rgb += boulderEnergyValue; 
    return OUT;
}

VertexOutput VS_Particle(VertexInput IN, VertexOutput OUT)
{	
	// WaveForm Calculations
	float waveDistance = abs( length(IN.position - WaveRadius) );		
	float waveForm = sqrt(waveDistance * abs( cos( waveDistance ) ) );
	
	// Don't modify verteces below a specific latitude
	if(waveForm > 3.0f) 
	{
		OUT.position = IN.position + waveForm * boulderLookVector; 
	}

    OUT.color.r				= waveForm;     

    return OUT;    
}

// Basic Lighting Shader
//--------------------------------------------------------------------------------------------------
VertexOutput VS_BasicLighting(VertexInput IN, VertexOutput OUT)
{	
	// Texturing
    OUT.texCoord			= IN.texCoord;

	// Positional Calculations
	OUT.worldPos			= mul(IN.position, World).xyz;					// World Space Position
	OUT.position			= mul(IN.position, WorldViewProj);				// Raster Position
	OUT.normal				= IN.normal;									// Normal vector   

	//Per Vertex Light Calculations
	OUT.lightVector1		= mul(float3(light0[0][0], light0[0][1], light0[0][2]), World).xyz;
	OUT.lightVector2		= mul(float3(light1[0][0], light1[0][1], light1[0][2]), World).xyz;

	// Per Player Per Vertex Calculations
	OUT.vertexToEyeVector	= normalize(EyePostion[3].xyz - OUT.worldPos);	// eye vector

    return OUT;    
}


//Render Terrain Shader
//--------------------------------------------------------------------------------------------------
VertexOutput VS_RenderTerrain(VertexInput IN, VertexOutput OUT)
{
	// Texturing
    OUT.texCoord			= IN.texCoord;

	// Positional Calculations
	OUT.worldPos			= mul(IN.position, World).xyz;					// World Space Position
	OUT.position			= mul(IN.position, WorldViewProj);				// Raster Position
	OUT.normal				= IN.normal;									// Normal vector   

	//Per Vertex Light Calculations
	OUT.lightVector1		= mul(float3(light0[0][0], light0[0][1], light0[0][2]), World).xyz;
	OUT.lightVector2		= mul(float3(light1[0][0], light1[0][1], light1[0][2]), World).xyz;

	// Per Player Per Vertex Calculations
	OUT.vertexToEyeVector	= normalize(EyePostion[3].xyz - OUT.worldPos);	// eye vector

    return OUT;  
}

FragmentOutput FS_RenderTerrain(FragmentInput IN, FragmentOutput OUT) 
{
	// Calculate the pixel to light source vectors
	IN.lightVector1				-= IN.worldPos;																// Vertex to Light source 1 vector 
	IN.lightVector2				-= IN.worldPos;																// Vertex to Light source 2 vector 
	
	// Bumpy Normal
	float4 bumpNorm= tex2D(bumpTextureSampler1,  IN.texCoord)   * 10;
	bumpNorm  = (bumpNorm  * 2) - 1;
	// Calculate each light source's halfway vector
	float3 halfwayVector1		= normalize(IN.vertexToEyeVector + IN.lightVector1);						// Halfway vector
  	float3 halfwayVector2		= normalize(IN.vertexToEyeVector + IN.lightVector2);						// Halfway vector

	// Calculate the diffuse and specular contributions
    float diffuse0				= max(0, dot(normalize(IN.normal), normalize(IN.lightVector1)));			// Diffuse component
    float diffuse1				= max(0, dot(normalize(IN.normal), normalize(IN.lightVector2)));			// Diffuse component
    float specular0				= diffuse0 * pow(dot(normalize(IN.normal), halfwayVector1), coreShininess);	// Specular component
    float specular1				= diffuse1 * pow(dot(normalize(IN.normal), halfwayVector2), coreShininess);	// Specular component

	// Calculate Ambient, Diffuse, and Specular color components
    float4 ambientColor = coreMaterialDiffuse	* (				float4(light0[1][0], light0[1][1], light0[1][2], light0[1][3]) +				float4(light1[1][0], light1[1][1], light1[1][2], light1[1][3]));
    float4 diffuseColor = coreMaterialDiffuse	* (diffuse0 *	float4(light0[2][0], light0[2][1], light0[2][2], light0[2][3]) + diffuse1 *		float4(light1[2][0], light1[2][1], light1[2][2], light1[2][3]) );
    float4 specularColor = coreMaterialSpecular * (specular0 *	float4(light0[3][0], light0[3][1], light0[3][2], light0[3][3]) + specular1 *	float4(light1[3][0], light1[3][1], light1[3][2], light1[3][3]) );

	// Get texture value
 	float4 lowTex =	tex2D(modelTextureSampler1, IN.texCoord  * 10); 	
	float4 middleTex = tex2D(modelTextureSampler2, IN.texCoord  * 10);
	float4 highTex = tex2D(modelTextureSampler3, IN.texCoord * 10);
	float4 modTex = tex2D(modelTextureSampler4, IN.texCoord * 60);
	float4 texFinal;
	if(IN.worldPos.y / (worldMaxY - worldMinY) < .5 )
	{
		texFinal = lerp(lowTex, middleTex,  (IN.worldPos.y / (worldMaxY - worldMinY)) * 2);
		 //texFinal = lowTex;
	}
	else
	{
		texFinal = lerp(middleTex, highTex,  ((IN.worldPos.y / (worldMaxY - worldMinY) -  .5))  * 2 );
		//texFinal = middleTex;
	}
	
	// Apply lighting to the color
	float angle = degrees(acos(dot(normalize(IN.normal), float3(0,1,0))));
	if(angle > 90)
	{
		angle = abs(angle - 180);
	}	
	if (angle > 80)
	{
		texFinal = lerp(texFinal ,  modTex, (angle/ 90) * .3);
	}
	if (angle > 60)
	{
		texFinal = lerp(texFinal ,  modTex, angle/ 90 * .15);
	}
	else 
	{
		texFinal = lerp(texFinal ,  modTex, (angle/ 90) * .05);
	}
 	OUT.color += (diffuseColor * texFinal) + (ambientColor * texFinal) + specularColor;

	return OUT;
}

// RenderToTexture Shader
//--------------------------------------------------------------------------------------------------
FragmentOutput FS_RenderToTexture(FragmentInput IN, FragmentOutput OUT)
{
    OUT.color = tex2D(modelTextureSampler1, IN.texCoord);    
    return OUT;
}
VertexOutput VS_RenderToTexture(VertexInput IN, VertexOutput OUT)
{
	OUT.position = mul(IN.position, WorldViewProj);
    OUT.texCoord = IN.texCoord;
    return OUT;    
}

//===================================================================================================
// MAIN
//===================================================================================================

// Particle Main
//--------------------------------------------------------------------------------------------------
VertexOutput ParticleVertexMain(VertexInput VS_IN)
{
	VertexOutput VS_OUT = (VertexOutput)0;
	//VS_OUT =	VS_FireAOE(VS_IN, VS_OUT);
	//VS_IN.position = VS_OUT.position;
	VS_OUT =	VS_BasicLighting(VS_IN, VS_OUT);
	return VS_OUT;
}
FragmentOutput ParticleFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
	FS_OUT =	FS_Particle(FS_IN, FS_OUT);	
	FS_OUT =	FS_BasicLighting(FS_IN, FS_OUT);
	return FS_OUT;
}

// Basic Lighting Main
//--------------------------------------------------------------------------------------------------
VertexOutput BasicLightingVertexMain(VertexInput VS_IN)
{
	VertexOutput VS_OUT = (VertexOutput)0;
	VS_OUT = VS_BasicLighting(VS_IN, VS_OUT);
	return VS_OUT;
}
FragmentOutput BasicLightingFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
	FS_OUT = FS_BasicLighting(FS_IN, FS_OUT);
	return FS_OUT;
}

// RenderToTexture Main
//--------------------------------------------------------------------------------------------------
VertexOutput RenderToTextureVertexMain(VertexInput VS_IN)
{
	VertexOutput VS_OUT = (VertexOutput)0;
	VS_OUT = VS_RenderToTexture(VS_IN, VS_OUT);
	return VS_OUT;
}
FragmentOutput RenderToTextureFragmentMain(FragmentInput FS_IN)
{
    FragmentOutput FS_OUT = (FragmentOutput)0;
	FS_OUT = FS_RenderToTexture(FS_IN, FS_OUT);
	return FS_OUT;
}

// RenderTerrainMain
//--------------------------------------------------------------------------------------------------
VertexOutput RenderTerrainVerrtexMain(VertexInput VS_IN)
{
	VertexOutput VS_OUT =  (VertexOutput)0;
	VS_OUT =  VS_RenderTerrain(VS_IN, VS_OUT);
	return VS_OUT;
}

FragmentOutput RenderTerrainFragmentMain(FragmentInput FS_IN)
{
	FragmentOutput FS_OUT = (FragmentOutput)0;
	FS_OUT = FS_RenderTerrain(FS_IN, FS_OUT);
	return FS_OUT;
}


//===================================================================================================
// Techniques
//===================================================================================================

// Particle Technique
//--------------------------------------------------------------------------------------------------
technique ParticleShaderMain
{
    pass Pass0
    {
        VertexShader = compile vs_2_0 ParticleVertexMain();
        PixelShader = compile ps_2_0 ParticleFragmentMain();
        
		// Alpha blending
        AlphaBlendEnable = true;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
                
		FillMode = Solid;
    }
}

// Basic Lighting Technique
//--------------------------------------------------------------------------------------------------
technique BasicLightingShaderMain
{
	pass Pass0
	{
		VertexShader = compile vs_2_0 BasicLightingVertexMain();
		PixelShader = compile ps_2_0 BasicLightingFragmentMain();
		
		FillMode = Solid;
	}
}

// RenderToTexture Technique
//--------------------------------------------------------------------------------------------------
technique RenderToTextureShaderMain
{
	pass Pass0
	{
		VertexShader = compile vs_2_0 RenderToTextureVertexMain();
		PixelShader = compile ps_2_0 RenderToTextureFragmentMain();
		
		FillMode = Solid;
	}
}

technique TerrainShaderMain
{
	pass Pass0
	{
		VertexShader  = compile vs_2_0 RenderTerrainVerrtexMain();
		PixelShader =  compile ps_2_b RenderTerrainFragmentMain();
		FillMode = Solid;
	}
}
